@@ -3,7 +3,11 @@
 # This demonstration script creates a canvas widet that displays the
 # floorplan for DEC's Western Research Laboratory.
 #
-# SCCS: @(#) floor.tcl 1.5 96/10/04 17:09:37
+# SCCS: @(#) floor.tcl 1.6 97/03/02 16:23:32
+
+if {![info exists widgetDemo]} {
+    error "This script should be run from the \"widget\" demo."
+}
 
 # floorDisplay --
 # Recreate the floorplan display in the canvas given by "w".  The
