@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMacCursor.c 1.10 96/02/15 18:55:44
+ * SCCS: @(#) tkMacCursor.c 1.17 96/12/17 15:23:07
  */
 
 #include "tkPort.h"
@@ -18,6 +18,13 @@
 #include <Resources.h>
 #include <ToolUtils.h>
 
+/*
+ * There are three different ways to set the cursor on the Mac.
+ */
+#define ARROW	0	/* The arrow cursor. */
+#define COLOR	1	/* Cursors of type crsr. */
+#define NORMAL	2	/* Cursors of type CURS. */
+
 /*
  * The following data structure contains the system specific data
  * necessary to control Windows cursors.
@@ -26,8 +33,7 @@
 typedef struct {
     TkCursor info;		/* Generic cursor info used by tkCursor.c */
     Handle macCursor;		/* Resource containing Macintosh cursor. */
-    int type;			/* Type of Mac cursor: 
-    				 * 0 = arrow, 1 = crsr, 2 = CURS */
+    int type;			/* Type of Mac cursor: arrow, crsr, CURS */
 } TkMacCursor;
 
 /*
@@ -51,6 +57,15 @@ static struct CursorName {
     {NULL,		0}
 };
 
+/*
+ * Declarations of static variables used in this file.
+ */
+
+static TkMacCursor *	gCurrentCursor = NULL;	 /* A pointer to the current
+						  * cursor. */
+static int		gResizeOverride = false; /* A boolean indicating wether
+						  * we should use the resize
+						  * cursor during installations. */
 
 /*
  *----------------------------------------------------------------------
@@ -69,14 +84,14 @@ static struct CursorName {
  */
 
 TkCursor *
-TkGetCursorByName(interp, tkwin, string)
-    Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
-    Tk_Window tkwin;		/* Window in which cursor will be used. */
-    Tk_Uid string;		/* Description of cursor.  See manual entry
+TkGetCursorByName(
+    Tcl_Interp *interp,		/* Interpreter to use for error reporting. */
+    Tk_Window tkwin,		/* Window in which cursor will be used. */
+    Tk_Uid string)		/* Description of cursor.  See manual entry
 				 * for details on legal syntax. */
 {
     struct CursorName *namePtr;
-    TkMacCursor *cursorPtr;
+    TkMacCursor *macCursorPtr;
 
     /*
      * Check for the cursor in the system cursor set.
@@ -88,15 +103,15 @@ TkGetCursorByName(interp, tkwin, string)
 	}
     }
 
-    cursorPtr = (TkMacCursor *) ckalloc(sizeof(TkMacCursor));
-    cursorPtr->info.cursor = (Tk_Cursor) cursorPtr;
+    macCursorPtr = (TkMacCursor *) ckalloc(sizeof(TkMacCursor));
+    macCursorPtr->info.cursor = (Tk_Cursor) macCursorPtr;
     if (namePtr->name != NULL) {
     	if (namePtr->id == 5) {
-	    cursorPtr->macCursor = (Handle) -1;
-	    cursorPtr->type = 0;
+	    macCursorPtr->macCursor = (Handle) -1;
+	    macCursorPtr->type = ARROW;
     	} else {
-	    cursorPtr->macCursor = (Handle) GetCursor(namePtr->id);
-	    cursorPtr->type = 2;
+	    macCursorPtr->macCursor = (Handle) GetCursor(namePtr->id);
+	    macCursorPtr->type = NORMAL;
 	}
     } else {
     	Handle resource;
@@ -113,22 +128,22 @@ TkGetCursorByName(interp, tkwin, string)
 	    HLock(resource);
 	    GetResInfo(resource, &id, &theType, theName);
 	    HUnlock(resource);
-	    cursorPtr->macCursor = (Handle) GetCCursor(id);
-	    cursorPtr->type = 1;
+	    macCursorPtr->macCursor = (Handle) GetCCursor(id);
+	    macCursorPtr->type = COLOR;
 	}
 
 	if (resource == NULL) {
-	    cursorPtr->macCursor = GetNamedResource('CURS', curName);
-	    cursorPtr->type = 2;
+	    macCursorPtr->macCursor = GetNamedResource('CURS', curName);
+	    macCursorPtr->type = NORMAL;
 	}
     }
-    if (cursorPtr->macCursor == NULL) {
-	ckfree((char *)cursorPtr);
+    if (macCursorPtr->macCursor == NULL) {
+	ckfree((char *)macCursorPtr);
 	Tcl_AppendResult(interp, "bad cursor spec \"", string, "\"",
 		(char *) NULL);
 	return NULL;
     } else {
-	return (TkCursor *) cursorPtr;
+	return (TkCursor *) macCursorPtr;
     }
 }
 
@@ -149,15 +164,14 @@ TkGetCursorByName(interp, tkwin, string)
  */
 
 TkCursor *
-TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
-	fgColor, bgColor)
-    Tk_Window tkwin;		/* Window in which cursor will be used. */
-    char *source;		/* Bitmap data for cursor shape. */
-    char *mask;			/* Bitmap data for cursor mask. */
-    int width, height;		/* Dimensions of cursor. */
-    int xHot, yHot;		/* Location of hot-spot in cursor. */
-    XColor fgColor;		/* Foreground color for cursor. */
-    XColor bgColor;		/* Background color for cursor. */
+TkCreateCursorFromData(
+    Tk_Window tkwin,		/* Window in which cursor will be used. */
+    char *source,		/* Bitmap data for cursor shape. */
+    char *mask,			/* Bitmap data for cursor mask. */
+    int width, int height,	/* Dimensions of cursor. */
+    int xHot, int yHot,		/* Location of hot-spot in cursor. */
+    XColor fgColor,		/* Foreground color for cursor. */
+    XColor bgColor)		/* Background color for cursor. */
 {
     return NULL;
 }
@@ -180,75 +194,101 @@ TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
  */
 
 void
-TkFreeCursor(cursorPtr)
-    TkCursor *cursorPtr;
+TkFreeCursor(
+    TkCursor *cursorPtr)
 {
     TkMacCursor *macCursorPtr = (TkMacCursor *) cursorPtr;
 
     switch (macCursorPtr->type) {
-	case 1:
+	case COLOR:
 	    DisposeCCursor((CCrsrHandle) macCursorPtr->macCursor);
 	    break;
-	case 2:
+	case NORMAL:
 	    ReleaseResource(macCursorPtr->macCursor);
 	    break;
     }
+
+    if (macCursorPtr == gCurrentCursor) {
+	gCurrentCursor = NULL;
+    }
+    
     ckfree((char *) macCursorPtr);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * TkUpdateCursor --
+ * TkMacInstallCursor --
  *
- *	Set the windows global cursor to the cursor associated with
- *	the given Tk window.
+ *	Installs either the current cursor as defined by TkpSetCursor
+ *	or a resize cursor as the cursor the Macintosh should currently
+ *	display.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	Changes the mouse cursor.
+ *	Changes the Macintosh mouse cursor.
  *
  *----------------------------------------------------------------------
  */
 
 void
-TkUpdateCursor(winPtr)
-    TkWindow *winPtr;
+TkMacInstallCursor(
+    int resizeOverride)
 {
-    TkMacCursor *macCursor = NULL;
+    TkMacCursor *macCursorPtr = gCurrentCursor;
+    CCrsrHandle ccursor;
+    CursHandle cursor;
 
-    /*
-     * A window inherits its cursor from its parent if it doesn't
-     * have one of its own.  Top level windows inherit the default
-     * cursor.
-     */
-
-    while (winPtr != NULL) {
-	if (winPtr->atts.cursor != None) {
-	    macCursor = (TkMacCursor *) winPtr->atts.cursor;
-	    break;
-	} else if (winPtr->flags & TK_TOP_LEVEL) {
-	    break;
-	}
-	winPtr = winPtr->parentPtr;
-    }
-    if (macCursor == NULL || macCursor->type == 0) {
-	SetCursor(&qd.arrow);
+    gResizeOverride = resizeOverride;
+    
+    if (resizeOverride) {
+	cursor = (CursHandle) GetNamedResource('CURS', "\presize");
+	SetCursor(*cursor);
+    } else if (macCursorPtr == NULL || macCursorPtr->type == ARROW) {
+	SetCursor(&tcl_macQdPtr->arrow);
     } else {
-	CCrsrHandle ccursor;
-	CursHandle cursor;
-	
-	switch (macCursor->type) {
-	    case 1:
-		ccursor = (CCrsrHandle) macCursor->macCursor;
+	switch (macCursorPtr->type) {
+	    case COLOR:
+		ccursor = (CCrsrHandle) macCursorPtr->macCursor;
 		SetCCursor(ccursor);
 		break;
-	    case 2:
-		cursor = (CursHandle) macCursor->macCursor;
+	    case NORMAL:
+		cursor = (CursHandle) macCursorPtr->macCursor;
 		SetCursor(*cursor);
 		break;
 	}
     }
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpSetCursor --
+ *
+ *	Set the current cursor and install it.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Changes the current cursor.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpSetCursor(
+    TkpCursor cursor)
+{
+    if (cursor == None) {
+	gCurrentCursor = NULL;
+    } else {
+	gCurrentCursor = (TkMacCursor *) cursor;
+    }
+
+    if (tkMacAppInFront) {
+	TkMacInstallCursor(gResizeOverride);
+    }
+}
