@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMacRegion.c 1.8 96/05/01 15:18:24
+ * SCCS: @(#) tkMacRegion.c 1.9 96/12/03 11:46:50
  */
 
 #include "tkInt.h"
@@ -68,8 +68,8 @@ TkCreateRegion()
  */
 
 void 
-TkDestroyRegion(r)
-    TkRegion r;
+TkDestroyRegion(
+    TkRegion r)
 {
     RgnHandle rgn = (RgnHandle) r;
 
@@ -94,10 +94,10 @@ TkDestroyRegion(r)
  */
 
 void 
-TkIntersectRegion(sra, srb, dr_return)
-    TkRegion sra;
-    TkRegion srb;
-    TkRegion dr_return;
+TkIntersectRegion(
+    TkRegion sra,
+    TkRegion srb,
+    TkRegion dr_return)
 {
     RgnHandle srcRgnA = (RgnHandle) sra;
     RgnHandle srcRgnB = (RgnHandle) srb;
@@ -125,10 +125,10 @@ TkIntersectRegion(sra, srb, dr_return)
  */
 
 void 
-TkUnionRectWithRegion(rectangle, src_region, dest_region_return)
-    XRectangle* rectangle;
-    TkRegion src_region;
-    TkRegion dest_region_return;
+TkUnionRectWithRegion(
+    XRectangle* rectangle,
+    TkRegion src_region,
+    TkRegion dest_region_return)
 {
     RgnHandle srcRgn = (RgnHandle) src_region;
     RgnHandle destRgn = (RgnHandle) dest_region_return;
@@ -159,12 +159,12 @@ TkUnionRectWithRegion(rectangle, src_region, dest_region_return)
  */
 
 int 
-TkRectInRegion(region, x, y, width, height)
-    TkRegion region;
-    int x;
-    int y;
-    unsigned int width;
-    unsigned int height;
+TkRectInRegion(
+    TkRegion region,
+    int x,
+    int y,
+    unsigned int width,
+    unsigned int height)
 {
     RgnHandle rgn = (RgnHandle) region;
     RgnHandle rectRgn, destRgn;
@@ -204,9 +204,9 @@ TkRectInRegion(region, x, y, width, height)
  */
 
 void 
-TkClipBox(r, rect_return)
-    TkRegion r;
-    XRectangle* rect_return;
+TkClipBox(
+    TkRegion r,
+    XRectangle* rect_return)
 {
     RgnHandle rgn = (RgnHandle) r;
 
