@@ -6,12 +6,12 @@
  * Macintosh or thier implamentation just doesn't do anything.  Other
  * calls will eventually be moved into other files.
  *
- * Copyright (c) 1995-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMacXStubs.c 1.73 97/01/17 16:01:20
+ * SCCS: @(#) tkMacXStubs.c 1.84 97/05/06 14:13:58
  */
 
 #include "tkInt.h"
@@ -20,7 +20,6 @@
 #include <stdio.h>
 #include <tcl.h>
 
-#include "xcolors.h"
 #include <Xatom.h>
 
 #include <Windows.h>
@@ -44,7 +43,7 @@
  * Declarations of static variables used in this file.
  */
 
-static Display *gMacDisplay = NULL; /* Macintosh display. */
+static TkDisplay *gMacDisplay = NULL; /* Macintosh display. */
 static char *macScreenName = "Macintosh:0";
 				/* Default name of macintosh display. */
 
@@ -68,13 +67,11 @@ XImage *TkMacXSubImage _ANSI_ARGS_((XImage *image, int x, int y,
 	unsigned int width, unsigned int height));
 int TkMacXAddPixel _ANSI_ARGS_((XImage *image, long value));
 int _XInitImageFuncPtrs _ANSI_ARGS_((XImage *image));
-void Tk_GetToplevelCoords _ANSI_ARGS_((Tk_Window tkwin, int *xPtr, int *yPtr));
-
 
 /*
  *----------------------------------------------------------------------
  *
- * XOpenDisplay --
+ * TkpOpenDisplay --
  *
  *	Create the Display structure and fill it with device
  *	specific information.
@@ -88,16 +85,16 @@ void Tk_GetToplevelCoords _ANSI_ARGS_((Tk_Window tkwin, int *xPtr, int *yPtr));
  *----------------------------------------------------------------------
  */
 
-Display *
-XOpenDisplay(display_name)
-    _Xconst char *display_name;
+TkDisplay *
+TkpOpenDisplay(
+    char *display_name)
 {
     Display *display;
     Screen *screen;
     GDHandle graphicsDevice;
 
     if (gMacDisplay != NULL) {
-	if (strcmp(gMacDisplay->display_name, display_name) == 0) {
+	if (strcmp(gMacDisplay->display->display_name, display_name) == 0) {
 	    return gMacDisplay;
 	} else {
 	    return NULL;
@@ -117,13 +114,13 @@ XOpenDisplay(display_name)
     
     screen->root = ROOT_ID;
     screen->display = display;
-    screen->root_depth = 64;	/* Todo: get max depth */
+    screen->root_depth = 24;	/* Todo: get max depth */
     screen->height = (*graphicsDevice)->gdRect.bottom -
 	(*graphicsDevice)->gdRect.top;
     screen->width = (*graphicsDevice)->gdRect.right -
 	(*graphicsDevice)->gdRect.left;
-    screen->mwidth = screen->width / 4;  /* TODO: determine resolution */
-    screen->mheight = screen->height / 4;
+    screen->mwidth = (screen->width * 254 + 360) / 720;
+    screen->mheight = (screen->height * 254 + 360) / 720;
     screen->black_pixel = 0x00000000;
     screen->white_pixel = 0x00FFFFFF;
     screen->root_visual = (Visual *) ckalloc(sizeof(Visual));
@@ -132,11 +129,55 @@ XOpenDisplay(display_name)
     screen->root_visual->red_mask = 0x00FF0000;
     screen->root_visual->green_mask = 0x0000FF00;
     screen->root_visual->blue_mask = 0x000000FF;
-    screen->root_visual->bits_per_rgb = 8;
+    screen->root_visual->bits_per_rgb = 24;
     screen->root_visual->map_entries = 2 ^ 8;
 
-    gMacDisplay = display;
-    return display;
+    gMacDisplay = (TkDisplay *) ckalloc(sizeof(TkDisplay));
+    gMacDisplay->display = display;
+    return gMacDisplay;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpCloseDisplay --
+ *
+ *	Deallocates a display structure created by TkpOpenDisplay.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Frees memory.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpCloseDisplay(
+    TkDisplay *displayPtr)
+{
+    Display *display = displayPtr->display;
+    if (gMacDisplay != displayPtr) {
+        panic("TkpCloseDisplay: tried to call TkpCloseDisplay on bad display");
+    }
+
+    /*
+     * Make sure that the local scrap is transfered to the global
+     * scrap if needed.
+     */
+
+    TkSuspendClipboard();
+
+    gMacDisplay = NULL;
+    if (display->screens != (Screen *) NULL) {
+        if (display->screens->root_visual != (Visual *) NULL) {
+            ckfree((char *) display->screens->root_visual);
+        }
+        ckfree((char *) display->screens);
+    }
+    ckfree((char *) display);
+    ckfree((char *) displayPtr);
 }
 
 /*
@@ -159,8 +200,8 @@ XOpenDisplay(display_name)
  */
 
 static XID
-MacXIdAlloc(display)
-    Display *display;			/* Display for which to allocate. */
+MacXIdAlloc(
+    Display *display)			/* Display for which to allocate. */
 {
 	static long int cur_id = 100;
 	/*
@@ -171,6 +212,33 @@ MacXIdAlloc(display)
 	return ++cur_id;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpWindowWasRecentlyDeleted --
+ *
+ *	Tries to determine whether the given window was recently deleted.
+ *	Called from the generic code error handler to attempt to deal with
+ *	async BadWindow errors under some circumstances.
+ *
+ * Results:
+ *	Always 0, we do not keep this information on the Mac, so we do not
+ *	know whether the window was destroyed.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TkpWindowWasRecentlyDeleted(
+    Window win,
+    TkDisplay *dispPtr)
+{
+    return 0;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -189,9 +257,9 @@ MacXIdAlloc(display)
  */
 
 static int
-DefaultErrorHandler(display, err_evt)
-    Display* display;
-    XErrorEvent* err_evt;
+DefaultErrorHandler(
+    Display* display,
+    XErrorEvent* err_evt)
 {
     /*
      * This call should never be called.  Tk replaces
@@ -203,9 +271,9 @@ DefaultErrorHandler(display, err_evt)
 
 
 char *
-XGetAtomName(display, atom)
-    Display * display;
-    Atom atom;
+XGetAtomName(
+    Display * display,
+    Atom atom)
 {
     display->request++;
     return NULL;
@@ -218,8 +286,8 @@ _XInitImageFuncPtrs(XImage *image)
 }
 
 XErrorHandler
-XSetErrorHandler(handler)
-    XErrorHandler handler;
+XSetErrorHandler(
+    XErrorHandler handler)
 {
     return DefaultErrorHandler;
 }
@@ -246,47 +314,6 @@ XGetImage(display, d, x, y, width, height, plane_mask, format)
     return NULL;
 }
 
-void
-XGetInputFocus(display, focus_return, revert_to_return)
-    Display* display;
-    Window* focus_return;
-    int* revert_to_return;
-{
-    WindowPtr frontWin;
-    Window window;
-    Tk_Window tkwin;
-
-    display->request++;
-    *focus_return = None;
-    *revert_to_return = RevertToNone;
-    
-    frontWin = FrontWindow();
-    if (frontWin != NULL) {
-	window = TkMacGetXWindow(frontWin);
-	tkwin = Tk_IdToWindow(display, window);
-	if (tkwin != NULL) {
-	    *focus_return = window;
-	}
-    }
-}
-
-Bool
-XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return,
-	dest_y_return, child_return)
-    Display* display;
-    Window src_w;
-    Window dest_w;
-    int src_x;
-    int src_y;
-    int* dest_x_return;
-    int* dest_y_return;
-    Window* child_return;
-{
-    /* Used in the wm code */
-    Debugger();
-    return 0;
-}
-
 int
 XGetGeometry(display, d, root_return, x_return, y_return, width_return,
 	height_return, border_width_return, depth_return)
@@ -306,109 +333,41 @@ XGetGeometry(display, d, root_return, x_return, y_return, width_return,
 }
 
 void
-XDeleteProperty(display, w, property)
-    Display* display;
-    Window w;
-    Atom property;
+XChangeProperty(
+    Display* display,
+    Window w,
+    Atom property,
+    Atom type,
+    int format,
+    int mode,
+    _Xconst unsigned char* data,
+    int nelements)
 {
     Debugger();
 }
 
 void
-XSetWMClientMachine(disp, w, prop)
-    Display *disp;
-    Window w;
-    XTextProperty *prop;
+XSelectInput(
+    Display* display,
+    Window w,
+    long event_mask)
 {
     Debugger();
 }
 
 void
-XSetCommand(display, w, argv, argc)
-    Display* display;
-    Window w;
-    char** argv;
-    int argc;
-{
-    Debugger();
-}
-
-void
-XChangeProperty(display, w, property, type, format, mode, data, nelements)
-    Display* display;
-    Window w;
-    Atom property;
-    Atom type;
-    int format;
-    int mode;
-    _Xconst unsigned char* data;
-    int nelements;
-{
-    Debugger();
-}
-
-void
-XSetTransientForHint(display, w, prop_window)
-    Display* display;
-    Window w;
-    Window prop_window;
-{
-    /*
-     * This is ignored for now.  Eventually this should be displayed
-     * as a modal dialog window type.  However, we should *really* redefine
-     * toplevel to take window types instead of doing all this weird
-     * stuff.
-     */
-}
-
-Status
-XStringListToTextProperty(list, count, text_prop_return)
-    char** list;
-    int count;
-    XTextProperty* text_prop_return;
-{
-    /* Used in wm code */
-    Debugger();
-    return 0;
-}
-
-void
-XSelectInput(display, w, event_mask)
-    Display* display;
-    Window w;
-    long event_mask;
-{
-    Debugger();
-}
-
-int
-XQueryTree(display, w, root_return, parent_return, children_return,
-	nchildren_return)
-    Display* display;
-    Window w;
-    Window* root_return;
-    Window* parent_return;
-    Window** children_return;
-    unsigned int* nchildren_return;
-{
-    /* Used in wm code */
-    Debugger();
-    return 0;
-}
-
-void
-XBell(display, percent)
-    Display* display;
-    int percent;
+XBell(
+    Display* display,
+    int percent)
 {
     SysBeep(percent);
 }
 
 void
-XSetWMNormalHints(display, w, hints)
-    Display* display;
-    Window w;
-    XSizeHints* hints;
+XSetWMNormalHints(
+    Display* display,
+    Window w,
+    XSizeHints* hints)
 {
     /*
      * Do nothing.  Shouldn't even be called.
@@ -426,33 +385,18 @@ XAllocSizeHints()
     return NULL;
 }
 
-void
-XSetInputFocus(display, focus, revert_to, time)
-    Display* display;
-    Window focus;
-    int revert_to;
-    Time time;
-{
-    /*
-     * Currently a no-op
-     * TODO: implement focus!
-     */
-    display->request++;
-}
-
 XImage * 
-XCreateImage(display, visual, depth, format, offset, data,
-			width, height, bitmap_pad, bytes_per_line)
-    Display* display;
-    Visual* visual;
-    unsigned int depth;
-    int format;
-    int offset;
-    char* data;
-    unsigned int width;
-    unsigned int height;
-    int bitmap_pad;
-    int bytes_per_line;
+XCreateImage(
+    Display* display,
+    Visual* visual,
+    unsigned int depth,
+    int format,
+    int offset,
+    char* data,
+    unsigned int width,
+    unsigned int height,
+    int bitmap_pad,
+    int bytes_per_line)
 { 
     XImage *ximage;
 
@@ -495,41 +439,39 @@ XCreateImage(display, visual, depth, format, offset, data,
 }
 
 GContext
-XGContextFromGC(gc)
-    GC gc;
+XGContextFromGC(
+    GC gc)
 {
     /* TODO - currently a no-op */
     return 0;
 }
 
 Status
-XSendEvent(display, w, propagate, event_mask, event_send)
-    Display* display;
-    Window w;
-    Bool propagate;
-    long event_mask;
-    XEvent* event_send;
+XSendEvent(
+    Display* display,
+    Window w,
+    Bool propagate,
+    long event_mask,
+    XEvent* event_send)
 {
     Debugger();
     return 0;
 }
 
 int
-XGetWindowProperty(display, w, property, long_offset, long_length, delete,
-	req_type, actual_type_return, actual_format_return, nitems_return,
-	bytes_after_return, prop_return)
-    Display *display;
-    Window w;
-    Atom property;
-    long long_offset;
-    long long_length;
-    Bool delete;
-    Atom req_type;
-    Atom *actual_type_return;
-    int *actual_format_return;
-    unsigned long *nitems_return;
-    unsigned long *bytes_after_return;
-    unsigned char ** prop_return;
+XGetWindowProperty(
+    Display *display,
+    Window w,
+    Atom property,
+    long long_offset,
+    long long_length,
+    Bool delete,
+    Atom req_type,
+    Atom *actual_type_return,
+    int *actual_format_return,
+    unsigned long *nitems_return,
+    unsigned long *bytes_after_return,
+    unsigned char ** prop_return)
 {
     display->request++;
     *actual_type_return = None;
@@ -545,60 +487,11 @@ XRefreshKeyboardMapping()
     Debugger();
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * Tk_GetRootCoords --
- *
- *	Given a token for a window, this procedure traces through the
- *	window's lineage to find the (virtual) root-window coordinates
- *	corresponding to point (0,0) in the window.
- *
- * Results:
- *	The locations pointed to by xPtr and yPtr are filled in with
- *	the root coordinates of the (0,0) point in tkwin.  If a virtual
- *	root window is in effect for the window, then the coordinates
- *	in the virtual root are returned.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-void
-Tk_GetToplevelCoords(tkwin, xPtr, yPtr)
-    Tk_Window tkwin;		/* Token for window. */
-    int *xPtr;			/* Where to store x-displacement of (0,0). */
-    int *yPtr;			/* Where to store y-displacement of (0,0). */
-{
-    int x, y;
-    register TkWindow *winPtr = (TkWindow *) tkwin;
-
-    /*
-     * Search back through this window's parents all the way to a
-     * top-level window, combining the offsets of each window within
-     * its parent.
-     */
-
-    x = y = 0;
-    while (1) {
-	if (winPtr->flags & TK_TOP_LEVEL) {
-	    break;
-	}
-	x += winPtr->changes.x + winPtr->changes.border_width;
-	y += winPtr->changes.y + winPtr->changes.border_width;
-	winPtr = winPtr->parentPtr;
-    }
-    *xPtr = x;
-    *yPtr = y;
-}
-
 void 
-XSetIconName(display, w, icon_name)
-    Display* display;
-    Window w;
-    const char icon_name;
+XSetIconName(
+    Display* display,
+    Window w,
+    const char *icon_name)
 {
     /*
      * This is a no-op, no icon name for Macs.
@@ -607,23 +500,9 @@ XSetIconName(display, w, icon_name)
 }
 
 void 
-XDefineCursor(display, w, cursor)
-    Display* display;
-    Window w;
-    Cursor cursor;
-{
-    /* 
-     * This function is just a no-op.  The cursor for a
-     * a given window is retrieved from the atts field
-     * of the windows winPtr data structure.
-     */
-    display->request++;
-}
-
-void 
-XForceScreenSaver(display, mode)
-    Display* display;
-    int mode;
+XForceScreenSaver(
+    Display* display,
+    int mode)
 {
     /* 
      * This function is just a no-op.  It is defined to 
@@ -652,10 +531,10 @@ XForceScreenSaver(display, mode)
  */
 
 void
-TkGetServerInfo(interp, tkwin)
-    Tcl_Interp *interp;		/* The server information is returned in
+TkGetServerInfo(
+    Tcl_Interp *interp,		/* The server information is returned in
 				 * this interpreter's result. */
-    Tk_Window tkwin;		/* Token for window;  this selects a
+    Tk_Window tkwin)		/* Token for window;  this selects a
 				 * particular display and server. */
 {
     char buffer[50], buffer2[50];
@@ -671,50 +550,50 @@ TkGetServerInfo(interp, tkwin)
  */
 
 int 
-TkMacXDestroyImage(image)
-    XImage *image;
+TkMacXDestroyImage(
+    XImage *image)
 {
     Debugger();
     return 0;
 }
 
 unsigned long 
-TkMacXGetPixel(image, x, y)
-    XImage *image;
-    int x; 
-    int y;
+TkMacXGetPixel(
+    XImage *image,
+    int x,
+    int y)
 {
     Debugger();
     return 0;
 }
 
 int 
-TkMacXPutPixel(image, x, y, pixel)
-    XImage *image;
-    int x;
-    int y;
-    unsigned long pixel;
+TkMacXPutPixel(
+    XImage *image,
+    int x,
+    int y,
+    unsigned long pixel)
 {
     /* Debugger(); */
     return 0;
 }
 
 XImage *
-TkMacXSubImage(image, x, y, width, height)
-    XImage *image;
-    int x;
-    int y;
-    unsigned int width;
-    unsigned int height;
+TkMacXSubImage(
+    XImage *image,
+    int x,
+    int y,
+    unsigned int width,
+    unsigned int height)
 {
     Debugger();
     return NULL;
 }
 
 int 
-TkMacXAddPixel(image, value)
-    XImage *image;
-    long value;
+TkMacXAddPixel(
+    XImage *image,
+    long value)
 {
     Debugger();
     return 0;
@@ -740,59 +619,59 @@ TkMacXAddPixel(image, value)
  */
 
 void
-XChangeWindowAttributes(display, w, value_mask, attributes)
-    Display* display;
-    Window w;
-    unsigned long value_mask;
-    XSetWindowAttributes* attributes;
+XChangeWindowAttributes(
+    Display* display,
+    Window w,
+    unsigned long value_mask,
+    XSetWindowAttributes* attributes)
 {
 }
 
 void 
-XSetWindowBackground(display, window, value)
-	Display *display;
-	Window window;
-	unsigned long value;
+XSetWindowBackground(
+	Display *display,
+	Window window,
+	unsigned long value)
 {
 }
 
 void
-XSetWindowBackgroundPixmap(display, w, background_pixmap)
-    Display* display;
-    Window w;
-    Pixmap background_pixmap;
+XSetWindowBackgroundPixmap(
+    Display* display,
+    Window w,
+    Pixmap background_pixmap)
 {
 }
 
 void
-XSetWindowBorder(display, w, border_pixel)
-    Display* display;
-    Window w;
-    unsigned long border_pixel;
+XSetWindowBorder(
+    Display* display,
+    Window w,
+    unsigned long border_pixel)
 {
 }
 
 void
-XSetWindowBorderPixmap(display, w, border_pixmap)
-    Display* display;
-    Window w;
-    Pixmap border_pixmap;
+XSetWindowBorderPixmap(
+    Display* display,
+    Window w,
+    Pixmap border_pixmap)
 {
 }
 
 void
-XSetWindowBorderWidth(display, w, width)
-    Display* display;
-    Window w;
-    unsigned int width;
+XSetWindowBorderWidth(
+    Display* display,
+    Window w,
+    unsigned int width)
 {
 }
 
 void
-XSetWindowColormap(display, w, colormap)
-    Display* display;
-    Window w;
-    Colormap colormap;
+XSetWindowColormap(
+    Display* display,
+    Window w,
+    Colormap colormap)
 {
     Debugger();
 }
@@ -815,9 +694,9 @@ XSetWindowColormap(display, w, colormap)
  */
 
 char *
-TkGetDefaultScreenName(interp, screenName)
-    Tcl_Interp *interp;		/* Not used. */
-    char *screenName;		/* If NULL, use default string. */
+TkGetDefaultScreenName(
+    Tcl_Interp *interp,		/* Not used. */
+    char *screenName)		/* If NULL, use default string. */
 {
     if ((screenName == NULL) || (screenName[0] == '\0')) {
 	screenName = macScreenName;
