@@ -10,7 +10,7 @@
 
 _XInitImageFuncPtrs(XImage *image)
 {
-    return NULL;
+    return 0;
 }
 
 /*
@@ -31,7 +31,7 @@ XStringListToTextProperty(list, count, text_prop_return)
     int count;
     XTextProperty* text_prop_return;
 {
-    return NULL;
+    return (Status) NULL;
 }
 
 /*
@@ -82,7 +82,7 @@ XCreatePixmapCursor(display, source, mask, foreground_color,
     unsigned int x;
     unsigned int y;
 {
-    return NULL;
+    return (Cursor) NULL;
 }
 
 void
@@ -104,7 +104,7 @@ XFilterEvent(event, window)
     XEvent* event;
     Window window;
 {
-    return NULL;
+    return 0;
 }
 
 extern void XForceScreenSaver(display, mode)
@@ -124,7 +124,7 @@ GContext
 XGContextFromGC(gc)
     GC gc;
 {
-    return NULL;
+    return (GContext) NULL;
 }
 
 char *
@@ -135,44 +135,13 @@ XGetAtomName(display, atom)
     return NULL;
 }
 
-int
-XGetGeometry(display, d, root_return, x_return, y_return, width_return,
-	height_return, border_width_return, depth_return)
-    Display* display;
-    Drawable d;
-    Window* root_return;
-    int* x_return;
-    int* y_return;
-    unsigned int* width_return;
-    unsigned int* height_return;
-    unsigned int* border_width_return;
-    unsigned int* depth_return;
-{
-    return NULL;
-}
-
-XImage *
-XGetImage(display, d, x, y, width, height, plane_mask, format)
-    Display* display;
-    Drawable d;
-    int x;
-    int y;
-    unsigned int width;
-    unsigned int height;
-    unsigned long plane_mask;
-    int	format;
-{
-    return NULL;
-}
-
-
 int
 XGetWindowAttributes(display, w, window_attributes_return)
     Display* display;
     Window w;
     XWindowAttributes* window_attributes_return;
 {
-    return NULL;
+    return 0;
 }
 
 Status
@@ -182,7 +151,7 @@ XGetWMColormapWindows(display, w, windows_return, count_return)
     Window** windows_return;
     int* count_return;
 {
-    return NULL;
+    return (Status) NULL;
 }
 
 int
@@ -191,7 +160,7 @@ XIconifyWindow(display, w, screen_number)
     Window w;
     int screen_number;
 {
-    return NULL;
+    return 0;
 }
 
 XHostAddress *
@@ -212,7 +181,7 @@ XLookupColor(display, colormap, color_name, exact_def_return,
     XColor* exact_def_return;
     XColor* screen_def_return;
 {
-    return NULL;
+    return 0;
 }
 
 void
@@ -248,7 +217,7 @@ XQueryTree(display, w, root_return, parent_return, children_return,
     Window** children_return;
     unsigned int* nchildren_return;
 {
-    return NULL;
+    return 0;
 }
 
 void
@@ -262,7 +231,7 @@ XRootWindow(display, screen_number)
     Display* display;
     int screen_number;
 {
-    return NULL;
+    return (Window) NULL;
 }
 
 void
@@ -281,7 +250,7 @@ XSendEvent(display, w, propagate, event_mask, event_send)
     long event_mask;
     XEvent* event_send;
 {
-    return NULL;
+    return 0;
 }
 
 void
@@ -308,24 +277,6 @@ XSetIconName(display, w, icon_name)
 {
 }
 
-void
-XSetTransientForHint(display, w, prop_window)
-    Display* display;
-    Window w;
-    Window prop_window;
-{
-}
-
-int
-XSetWMColormapWindows(display, w, colormap_windows, count)
-    Display* display;
-    Window w;
-    Window* colormap_windows;
-    int count;
-{
-    return 0;
-}
-
 void
 XSetWindowBackground(display, w, background_pixel)
     Display* display;
@@ -442,5 +393,5 @@ XGetWindowProperty(display, w, property, long_offset, long_length, delete,
     *nitems_return = 0;
     *bytes_after_return = 0;
     *prop_return = NULL;
-    return 0;			/* failure */
+    return BadValue;
 }
