@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkWinCursor.c 1.6 96/02/15 18:56:06
+ * SCCS: @(#) tkWinCursor.c 1.9 96/10/31 10:41:10
  */
 
 #include "tkWinInt.h"
@@ -100,10 +100,12 @@ TkGetCursorByName(interp, tkwin, string)
 
     cursorPtr = (TkWinCursor *) ckalloc(sizeof(TkWinCursor));
     cursorPtr->info.cursor = (Tk_Cursor) cursorPtr;
+    cursorPtr->winCursor = NULL;
     if (namePtr->name != NULL) {
 	cursorPtr->winCursor = LoadCursor(NULL, namePtr->id);
 	cursorPtr->system = 1;
-    } else {
+    }
+    if (cursorPtr->winCursor == NULL) {
 	cursorPtr->winCursor = LoadCursor(TkWinGetTkModule(), string);
 	cursorPtr->system = 0;
     }
@@ -175,10 +177,10 @@ TkFreeCursor(cursorPtr)
 /*
  *----------------------------------------------------------------------
  *
- * TkWinUpdateCursor --
+ * TkpSetCursor --
  *
- *	Set the windows global cursor to the cursor associated with
- *	the given Tk window.
+ *	Set the global cursor.  If the cursor is None, then use the
+ *	default Tk cursor.
  *
  * Results:
  *	None.
@@ -190,28 +192,19 @@ TkFreeCursor(cursorPtr)
  */
 
 void
-TkWinUpdateCursor(winPtr)
-    TkWindow *winPtr;
+TkpSetCursor(cursor)
+    TkpCursor cursor;
 {
-    HCURSOR cursor = NULL;
+    HCURSOR hcursor;
+    TkWinCursor *winCursor = (TkWinCursor *) cursor;
 
-    /*
-     * A window inherits its cursor from its parent if it doesn't
-     * have one of its own.  Top level windows inherit the default
-     * cursor.
-     */
-
-    while (winPtr != NULL) {
-	if (winPtr->atts.cursor != None) {
-	    cursor = ((TkWinCursor *) winPtr->atts.cursor)->winCursor;
-	    break;
-	} else if (winPtr->flags & TK_TOP_LEVEL) {
-	    cursor = LoadCursor(NULL, TK_DEFAULT_CURSOR);
-	    break;
-	}
-	winPtr = winPtr->parentPtr;
+    if (winCursor == NULL || winCursor->winCursor == NULL) {
+	hcursor = LoadCursor(NULL, TK_DEFAULT_CURSOR);
+    } else {
+	hcursor = winCursor->winCursor;
     }
-    if (cursor != NULL) {
-	SetCursor(cursor);
+
+    if (hcursor != NULL) {
+	SetCursor(hcursor);
     }
 }
